@@ -66,6 +66,10 @@ class MockImageExistsException(MockException):
     """Used as mock for rbd.ImageExists."""
 
 
+class MockOSErrorException(MockException):
+    """Used as mock for rbd.OSError."""
+
+
 def common_mocks(f):
     """Decorator to set mocks common to all tests.
 
@@ -135,6 +139,19 @@ CEPH_MON_DUMP = r"""dumped monmap epoch 1
 """
 
 
+class MockDriverConfig(object):
+    def __init__(self, **kwargs):
+        my_dict = vars(self)
+        my_dict.update(kwargs)
+        my_dict.setdefault('max_over_subscription_ratio', 1.0)
+        my_dict.setdefault('reserved_percentage', 0)
+        my_dict.setdefault('volume_backend_name', 'RBD')
+        my_dict.setdefault('_default', None)
+
+    def __call__(self, value):
+        return getattr(self, value, self._default)
+
+
 def mock_driver_configuration(value):
     if value == 'max_over_subscription_ratio':
         return 1.0
@@ -1152,8 +1169,9 @@ class RBDTestCase(test.TestCase):
             expected['replication_targets'] = [t['backend_id']for t in targets]
             expected['replication_targets'].append('default')
 
+        my_safe_get = MockDriverConfig(rbd_exclusive_cinder_pool=False)
         self.mock_object(self.driver.configuration, 'safe_get',
-                         mock_driver_configuration)
+                         my_safe_get)
 
         actual = self.driver.get_volume_stats(True)
         self.assertDictEqual(expected, actual)
@@ -1161,9 +1179,38 @@ class RBDTestCase(test.TestCase):
     @common_mocks
     @mock.patch('cinder.volume.drivers.rbd.RBDDriver._get_usage_info')
     @mock.patch('cinder.volume.drivers.rbd.RBDDriver._get_pool_stats')
-    def test_update_volume_stats_error(self, stats_mock, usage_mock):
+    def test_update_volume_stats_exclusive_pool(self, stats_mock, usage_mock):
+        stats_mock.return_value = (mock.sentinel.free_capacity_gb,
+                                   mock.sentinel.total_capacity_gb)
+
+        expected = dict(
+            volume_backend_name='RBD',
+            replication_enabled=False,
+            vendor_name='Open Source',
+            driver_version=self.driver.VERSION,
+            storage_protocol='ceph',
+            total_capacity_gb=mock.sentinel.total_capacity_gb,
+            free_capacity_gb=mock.sentinel.free_capacity_gb,
+            reserved_percentage=0,
+            thin_provisioning_support=True,
+            max_over_subscription_ratio=1.0,
+            multiattach=False)
+
+        my_safe_get = MockDriverConfig(rbd_exclusive_cinder_pool=True)
         self.mock_object(self.driver.configuration, 'safe_get',
-                         mock_driver_configuration)
+                         my_safe_get)
+
+        actual = self.driver.get_volume_stats(True)
+        self.assertDictEqual(expected, actual)
+        usage_mock.assert_not_called()
+
+    @common_mocks
+    @mock.patch('cinder.volume.drivers.rbd.RBDDriver._get_usage_info')
+    @mock.patch('cinder.volume.drivers.rbd.RBDDriver._get_pool_stats')
+    def test_update_volume_stats_error(self, stats_mock, usage_mock):
+        my_safe_get = MockDriverConfig(rbd_exclusive_cinder_pool=False)
+        self.mock_object(self.driver.configuration, 'safe_get',
+                         my_safe_get)
 
         expected = dict(volume_backend_name='RBD',
                         replication_enabled=False,
@@ -1174,7 +1221,6 @@ class RBDTestCase(test.TestCase):
                         free_capacity_gb='unknown',
                         reserved_percentage=0,
                         multiattach=False,
-                        provisioned_capacity_gb=0,
                         max_over_subscription_ratio=1.0,
                         thin_provisioning_support=True)
 
@@ -1570,6 +1616,64 @@ class RBDTestCase(test.TestCase):
 
         self.assertEqual(1, mock_driver._disconnect_from_rados.call_count)
 
+    def test_rbd_volume_proxy_external_conn(self):
+        mock_driver = mock.Mock(name='driver')
+        mock_driver._connect_to_rados.return_value = (None, None)
+        with driver.RBDVolumeProxy(mock_driver, self.volume_a.name,
+                                   client='fake_cl', ioctx='fake_io'):
+
+            mock_driver._connect_to_rados.assert_not_called()
+
+        mock_driver._disconnect_from_rados.assert_not_called()
+
+    def test_rbd_volume_proxy_external_conn_no_iocxt(self):
+        mock_driver = mock.Mock(name='driver')
+        mock_driver._connect_to_rados.return_value = ('fake_cl', 'fake_io')
+        with driver.RBDVolumeProxy(mock_driver, self.volume_a.name,
+                                   client='fake_cl', pool='vol_pool'):
+            mock_driver._connect_to_rados.assert_called_once_with(
+                'vol_pool', None, None)
+
+        mock_driver._disconnect_from_rados.assert_called_once_with(
+            'fake_cl', 'fake_io')
+
+    def test_rbd_volume_proxy_external_conn_error(self):
+        mock_driver = mock.Mock(name='driver')
+        mock_driver._connect_to_rados.return_value = (None, None)
+
+        class RBDError(Exception):
+            pass
+
+        mock_driver.rbd.Error = RBDError
+        mock_driver.rbd.Image.side_effect = RBDError()
+
+        self.assertRaises(RBDError, driver.RBDVolumeProxy,
+                          mock_driver, self.volume_a.name,
+                          client='fake_cl', ioctx='fake_io')
+
+        mock_driver._connect_to_rados.assert_not_called()
+        mock_driver._disconnect_from_rados.assert_not_called()
+
+    def test_rbd_volume_proxy_conn_error(self):
+        mock_driver = mock.Mock(name='driver')
+        mock_driver._connect_to_rados.return_value = (
+            'fake_client', 'fake_ioctx')
+
+        class RBDError(Exception):
+            pass
+
+        mock_driver.rbd.Error = RBDError
+        mock_driver.rbd.Image.side_effect = RBDError()
+
+        self.assertRaises(RBDError, driver.RBDVolumeProxy,
+                          mock_driver, self.volume_a.name,
+                          pool='fake-volumes')
+
+        mock_driver._connect_to_rados.assert_called_once_with(
+            'fake-volumes', None, None)
+        mock_driver._disconnect_from_rados.assert_called_once_with(
+            'fake_client', 'fake_ioctx')
+
     @common_mocks
     def test_connect_to_rados(self):
         # Default
@@ -1835,17 +1939,24 @@ class RBDTestCase(test.TestCase):
             return mock.Mock(return_value=mock.Mock(
                 size=mock.Mock(side_effect=(size_or_exc,))))
 
-        volumes = ['volume-1', 'non-existent', 'non-cinder-volume']
+        volumes = [
+            'volume-1',
+            'non-existent',
+            'non-existent',
+            'non-cinder-volume'
+        ]
 
         client = client_mock.return_value.__enter__.return_value
         rbdproxy_mock.return_value.list.return_value = volumes
 
         with mock.patch.object(self.driver, 'rbd',
-                               ImageNotFound=MockImageNotFoundException):
+                               ImageNotFound=MockImageNotFoundException,
+                               OSError=MockOSErrorException):
             volproxy_mock.side_effect = [
                 mock.MagicMock(**{'__enter__': FakeVolProxy(s)})
                 for s in (1.0 * units.Gi,
                           self.driver.rbd.ImageNotFound,
+                          self.driver.rbd.OSError,
                           2.0 * units.Gi)
             ]
             total_provision = self.driver._get_usage_info()
@@ -1853,7 +1964,8 @@ class RBDTestCase(test.TestCase):
         rbdproxy_mock.return_value.list.assert_called_once_with(client.ioctx)
 
         expected_volproxy_calls = [
-            mock.call(self.driver, v, read_only=True)
+            mock.call(self.driver, v, read_only=True,
+                      client=client.cluster, ioctx=client.ioctx)
             for v in volumes]
         self.assertEqual(expected_volproxy_calls, volproxy_mock.mock_calls)
 
