@@ -77,14 +77,18 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             self.failed_over_backend_name or self.backend_name)
         self.vserver = self.zapi_client.vserver
 
-        # Performance monitoring library
-        self.perf_library = perf_cmode.PerformanceCmodeLibrary(
-            self.zapi_client)
-
         # Storage service catalog
         self.ssc_library = capabilities.CapabilitiesLibrary(
             'nfs', self.vserver, self.zapi_client, self.configuration)
 
+        self.ssc_library.check_api_permissions()
+        self.using_cluster_credentials = (
+            self.ssc_library.cluster_user_supported())
+
+        # Performance monitoring library
+        self.perf_library = perf_cmode.PerformanceCmodeLibrary(
+            self.zapi_client)
+
     def _update_zapi_client(self, backend_name):
         """Set cDOT API client for the specified config backend stanza name."""
 
@@ -98,7 +102,6 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
     @utils.trace_method
     def check_for_setup_error(self):
         """Check that the driver is working and can communicate."""
-        self.ssc_library.check_api_permissions()
         self._add_looping_tasks()
         super(NetAppCmodeNfsDriver, self).check_for_setup_error()
 
@@ -143,12 +146,12 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
     def _handle_housekeeping_tasks(self):
         """Handle various cleanup activities."""
-
-        # Harvest soft-deleted QoS policy groups
-        self.zapi_client.remove_unused_qos_policy_groups()
-
         active_backend = self.failed_over_backend_name or self.backend_name
 
+        # Add the task that harvests soft-deleted QoS policy groups.
+        if self.using_cluster_credentials:
+            self.zapi_client.remove_unused_qos_policy_groups()
+
         LOG.debug("Current service state: Replication enabled: %("
                   "replication)s. Failed-Over: %(failed)s. Active Backend "
                   "ID: %(active)s",
@@ -252,12 +255,18 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         if not ssc:
             return pools
 
-        # Get up-to-date node utilization metrics just once
-        self.perf_library.update_performance_cache(ssc)
+        # Utilization and performance metrics require cluster-scoped
+        # credentials
+        if self.using_cluster_credentials:
+            # Get up-to-date node utilization metrics just once
+            self.perf_library.update_performance_cache(ssc)
 
-        # Get up-to-date aggregate capacities just once
-        aggregates = self.ssc_library.get_ssc_aggregates()
-        aggr_capacities = self.zapi_client.get_aggregate_capacities(aggregates)
+            # Get up-to-date aggregate capacities just once
+            aggregates = self.ssc_library.get_ssc_aggregates()
+            aggr_capacities = self.zapi_client.get_aggregate_capacities(
+                aggregates)
+        else:
+            aggr_capacities = {}
 
         for ssc_vol_name, ssc_vol_info in ssc.items():
 
@@ -267,7 +276,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             pool.update(ssc_vol_info)
 
             # Add driver capabilities and config info
-            pool['QoS_support'] = True
+            pool['QoS_support'] = self.using_cluster_credentials
             pool['consistencygroup_support'] = True
             pool['consistent_group_snapshot_enabled'] = True
             pool['multiattach'] = False
@@ -277,8 +286,11 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             capacity = self._get_share_capacity_info(nfs_share)
             pool.update(capacity)
 
-            dedupe_used = self.zapi_client.get_flexvol_dedupe_used_percent(
-                ssc_vol_name)
+            if self.using_cluster_credentials:
+                dedupe_used = self.zapi_client.get_flexvol_dedupe_used_percent(
+                    ssc_vol_name)
+            else:
+                dedupe_used = 0.0
             pool['netapp_dedupe_used_percent'] = na_utils.round_down(
                 dedupe_used)
 
@@ -437,8 +449,11 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                               '%s was unsuccessful.', volume['id'])
 
     def _delete_file(self, file_id, file_name):
-        (_vserver, flexvol) = self._get_export_ip_path(volume_id=file_id)
-        path_on_backend = '/vol' + flexvol + '/' + file_name
+        (host_ip, junction_path) = self._get_export_ip_path(volume_id=file_id)
+        vserver = self._get_vserver_for_ip(host_ip)
+        flexvol = self.zapi_client.get_vol_by_junc_vserver(
+            vserver, junction_path)
+        path_on_backend = '/vol/' + flexvol + '/' + file_name
         LOG.debug('Attempting to delete file %(path)s for ID %(file_id)s on '
                   'backend.', {'path': path_on_backend, 'file_id': file_id})
         self.zapi_client.delete_file(path_on_backend)
@@ -464,39 +479,6 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 LOG.exception('Exec of "rm" command on backing file for'
                               ' %s was unsuccessful.', snapshot['id'])
 
-    @utils.trace_method
-    def copy_image_to_volume(self, context, volume, image_service, image_id):
-        """Fetch the image from image_service and write it to the volume."""
-        copy_success = False
-        try:
-            major, minor = self.zapi_client.get_ontapi_version()
-            col_path = self.configuration.netapp_copyoffload_tool_path
-            # Search the local image cache before attempting copy offload
-            cache_result = self._find_image_in_cache(image_id)
-            if cache_result:
-                copy_success = self._copy_from_cache(volume, image_id,
-                                                     cache_result)
-                if copy_success:
-                    LOG.info('Copied image %(img)s to volume %(vol)s '
-                             'using local image cache.',
-                             {'img': image_id, 'vol': volume['id']})
-            # Image cache was not present, attempt copy offload workflow
-            if (not copy_success and col_path and
-                    major == 1 and minor >= 20):
-                LOG.debug('No result found in image cache')
-                self._copy_from_img_service(context, volume, image_service,
-                                            image_id)
-                LOG.info('Copied image %(img)s to volume %(vol)s using'
-                         ' copy offload workflow.',
-                         {'img': image_id, 'vol': volume['id']})
-                copy_success = True
-        except Exception:
-            LOG.exception('Copy offload workflow unsuccessful.')
-        finally:
-            if not copy_success:
-                super(NetAppCmodeNfsDriver, self).copy_image_to_volume(
-                    context, volume, image_service, image_id)
-
     def _get_ip_verify_on_cluster(self, host):
         """Verifies if host on same cluster and returns ip."""
         ip = na_utils.resolve_hostname(host)
@@ -508,13 +490,13 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
     def _copy_from_cache(self, volume, image_id, cache_result):
         """Try copying image file_name from cached file_name."""
-        LOG.debug("Trying copy from cache using copy offload.")
         copied = False
         cache_copy, found_local = self._find_image_location(cache_result,
-                                                            volume['id'])
+                                                            volume)
 
         try:
             if found_local:
+                LOG.debug("Trying copy from cache using cloning.")
                 (nfs_share, file_name) = cache_copy
                 self._clone_file_dst_exists(
                     nfs_share, file_name, volume['name'], dest_exists=True)
@@ -523,17 +505,14 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 copied = True
             elif (cache_copy and
                   self.configuration.netapp_copyoffload_tool_path):
+                LOG.debug("Trying copy from cache using copy offload.")
                 self._copy_from_remote_cache(volume, image_id, cache_copy)
                 copied = True
-
-            if copied:
-                self._post_clone_image(volume)
-
         except Exception:
             LOG.exception('Error in workflow copy from cache.')
         return copied
 
-    def _find_image_location(self, cache_result, volume_id):
+    def _find_image_location(self, cache_result, volume):
         """Finds the location of a cached image.
 
         Returns image location local to the NFS share, that matches the
@@ -543,7 +522,10 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
         found_local_copy = False
         cache_copy = None
-        provider_location = self._get_provider_location(volume_id)
+
+        provider_location = volume_utils.extract_host(volume['host'],
+                                                      level='pool')
+
         for res in cache_result:
             (share, file_name) = res
             if share == provider_location:
@@ -581,10 +563,11 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         return src_ip, src_path
 
     def _get_destination_ip_and_path(self, volume):
-        dest_ip = self._get_ip_verify_on_cluster(
-            self._get_host_ip(volume['id']))
-        dest_path = os.path.join(self._get_export_path(
-            volume['id']), volume['name'])
+        share = volume_utils.extract_host(volume['host'], level='pool')
+        share_ip_and_path = share.split(":")
+        dest_ip = self._get_ip_verify_on_cluster(share_ip_and_path[0])
+        dest_path = os.path.join(share_ip_and_path[1], volume['name'])
+
         return dest_ip, dest_path
 
     def _clone_file_dst_exists(self, share, src_name, dst_name,
@@ -597,11 +580,14 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
     def _copy_from_img_service(self, context, volume, image_service,
                                image_id):
         """Copies from the image service using copy offload."""
+
         LOG.debug("Trying copy from image service using copy offload.")
         image_loc = image_service.get_location(context, image_id)
         locations = self._construct_image_nfs_url(image_loc)
         src_ip = None
         selected_loc = None
+        cloned = False
+
         # this will match the first location that has a valid IP on cluster
         for location in locations:
             conn, dr = self._check_get_nfs_path_segs(location)
@@ -616,32 +602,31 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             raise exception.NotFound(_("Source host details not found."))
         (__, ___, img_file) = selected_loc.rpartition('/')
         src_path = os.path.join(dr, img_file)
-        dst_ip = self._get_ip_verify_on_cluster(self._get_host_ip(
-            volume['id']))
+
+        dst_ip, vol_path = self._get_destination_ip_and_path(volume)
+        share_path = vol_path.rsplit("/", 1)[0]
+        dst_share = dst_ip + ':' + share_path
+
         # tmp file is required to deal with img formats
         tmp_img_file = six.text_type(uuid.uuid4())
         col_path = self.configuration.netapp_copyoffload_tool_path
         img_info = image_service.show(context, image_id)
-        dst_share = self._get_provider_location(volume['id'])
         self._check_share_can_hold_size(dst_share, img_info['size'])
         run_as_root = self._execute_as_root
 
         dst_dir = self._get_mount_point_for_share(dst_share)
         dst_img_local = os.path.join(dst_dir, tmp_img_file)
+
         try:
-            # If src and dst share not equal
-            if (('%s:%s' % (src_ip, dr)) !=
-                    ('%s:%s' % (dst_ip, self._get_export_path(volume['id'])))):
-                dst_img_serv_path = os.path.join(
-                    self._get_export_path(volume['id']), tmp_img_file)
-                # Always run copy offload as regular user, it's sufficient
-                # and rootwrap doesn't allow copy offload to run as root
-                # anyways.
-                self._execute(col_path, src_ip, dst_ip, src_path,
-                              dst_img_serv_path, run_as_root=False,
-                              check_exit_code=0)
-            else:
-                self._clone_file_dst_exists(dst_share, img_file, tmp_img_file)
+            dst_img_serv_path = os.path.join(
+                share_path, tmp_img_file)
+            # Always run copy offload as regular user, it's sufficient
+            # and rootwrap doesn't allow copy offload to run as root
+            # anyways.
+            self._execute(col_path, src_ip, dst_ip, src_path,
+                          dst_img_serv_path, run_as_root=False,
+                          check_exit_code=0)
+
             self._discover_file_till_timeout(dst_img_local, timeout=120)
             LOG.debug('Copied image %(img)s to tmp file %(tmp)s.',
                       {'img': image_id, 'tmp': tmp_img_file})
@@ -683,11 +668,13 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 finally:
                     if os.path.exists(dst_img_conv_local):
                         self._delete_file_at_path(dst_img_conv_local)
-            self._post_clone_image(volume)
+            cloned = True
         finally:
             if os.path.exists(dst_img_local):
                 self._delete_file_at_path(dst_img_local)
 
+        return cloned
+
     @utils.trace_method
     def unmanage(self, volume):
         """Removes the specified volume from Cinder management.
