@@ -20,7 +20,7 @@ import inspect
 import json
 import os
 import re
-import tempfile
+import shutil
 import time
 
 from oslo_config import cfg
@@ -29,14 +29,17 @@ from oslo_utils import units
 import six
 
 from cinder import compute
+from cinder import context
 from cinder import coordination
 from cinder import db
 from cinder import exception
+from cinder.i18n import _
+from cinder.image import image_utils
 from cinder.objects import fields
 from cinder import utils
-from cinder.i18n import _, _LE, _LI, _LW
-from cinder.image import image_utils
+from cinder.volume import configuration
 from cinder.volume import driver
+from cinder.volume import utils as volume_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -97,8 +100,8 @@ volume_opts = [
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(nas_opts)
-CONF.register_opts(volume_opts)
+CONF.register_opts(nas_opts, group=configuration.SHARED_CONF_GROUP)
+CONF.register_opts(volume_opts, group=configuration.SHARED_CONF_GROUP)
 
 
 # TODO(bluex): remove when drivers stop using it
@@ -139,20 +142,51 @@ class RemoteFSDriver(driver.BaseVD):
     driver_volume_type = None
     driver_prefix = 'remotefs'
     volume_backend_name = None
+    vendor_name = 'Open Source'
     SHARE_FORMAT_REGEX = r'.+:/.+'
 
+    # We let the drivers inheriting this specify
+    # whether thin provisioning is supported or not.
+    _thin_provisioning_support = False
+
     def __init__(self, *args, **kwargs):
         super(RemoteFSDriver, self).__init__(*args, **kwargs)
         self.shares = {}
         self._mounted_shares = []
         self._execute_as_root = True
-        self._is_voldb_empty_at_startup = kwargs.pop('is_vol_db_empty', None)
+        self._is_voldb_empty_at_startup = self._check_if_volume_db_is_empty()
         self._supports_encryption = False
 
         if self.configuration:
             self.configuration.append_config_values(nas_opts)
             self.configuration.append_config_values(volume_opts)
 
+    def _set_voldb_empty_at_startup_indicator(self, ctxt):
+        """Determine if the Cinder volume DB is empty.
+
+        A check of the volume DB is done to determine whether it is empty or
+        not at this point.
+
+        :param ctxt: our working context
+        """
+        if not self.db:
+            return False
+        vol_entries = self.db.volume_get_all(ctxt, None, 1, filters=None)
+
+        if len(vol_entries) == 0:
+            LOG.info("Determined volume DB was empty at startup.")
+            return True
+        else:
+            LOG.info("Determined volume DB was not empty at startup.")
+            return False
+
+    def _check_if_volume_db_is_empty(self):
+        vol_db_empty = self._set_voldb_empty_at_startup_indicator(
+            context.get_admin_context())
+        LOG.debug("Cinder Volume DB check: vol_db_empty=%s", vol_db_empty)
+
+        return vol_db_empty
+
     def check_for_setup_error(self):
         """Just to override parent behavior."""
         pass
@@ -227,6 +261,23 @@ class RemoteFSDriver(driver.BaseVD):
                   " mount_point_base.")
         return None
 
+    @staticmethod
+    def _validate_state(current_state,
+                        acceptable_states,
+                        obj_description='volume',
+                        invalid_exc=exception.InvalidVolume):
+        if current_state not in acceptable_states:
+            message = _('Invalid %(obj_description)s state. '
+                        'Acceptable states for this operation: '
+                        '%(acceptable_states)s. '
+                        'Current %(obj_description)s state: '
+                        '%(current_state)s.')
+            raise invalid_exc(
+                message=message %
+                dict(obj_description=obj_description,
+                     acceptable_states=acceptable_states,
+                     current_state=current_state))
+
     @utils.trace
     def create_volume(self, volume):
         """Creates a volume.
@@ -242,9 +293,9 @@ class RemoteFSDriver(driver.BaseVD):
         LOG.debug('Creating volume %(vol)s', {'vol': volume.id})
         self._ensure_shares_mounted()
 
-        volume.provider_location = self._find_share(volume.size)
+        volume.provider_location = self._find_share(volume)
 
-        LOG.info(_LI('casted to %s'), volume.provider_location)
+        LOG.info('casted to %s', volume.provider_location)
 
         self._do_create_volume(volume)
 
@@ -284,7 +335,7 @@ class RemoteFSDriver(driver.BaseVD):
                 self._ensure_share_mounted(share)
                 mounted_shares.append(share)
             except Exception as exc:
-                LOG.error(_LE('Exception during mounting %s'), exc)
+                LOG.error('Exception during mounting %s', exc)
 
         self._mounted_shares = mounted_shares
 
@@ -300,9 +351,9 @@ class RemoteFSDriver(driver.BaseVD):
         LOG.debug('Deleting volume %(vol)s, provider_location: %(loc)s',
                   {'vol': volume.id, 'loc': volume.provider_location})
         if not volume.provider_location:
-            LOG.warning(_LW('Volume %s does not have '
-                            'provider_location specified, '
-                            'skipping'), volume.name)
+            LOG.warning('Volume %s does not have '
+                        'provider_location specified, '
+                        'skipping', volume.name)
             return
 
         self._ensure_share_mounted(volume.provider_location)
@@ -356,11 +407,6 @@ class RemoteFSDriver(driver.BaseVD):
                       'count=%d' % block_count,
                       run_as_root=self._execute_as_root)
 
-    def _fallocate(self, path, size):
-        """Creates a raw file of given size in GiB using fallocate."""
-        self._execute('fallocate', '--length=%sG' % size,
-                      path, run_as_root=self._execute_as_root)
-
     def _create_qcow2_file(self, path, size_gb):
         """Creates a QCOW2 file of a given size in GiB."""
 
@@ -385,8 +431,8 @@ class RemoteFSDriver(driver.BaseVD):
                       {'path': path, 'permissions': permissions})
         else:
             permissions = 'ugo+rw'
-            LOG.warning(_LW('%(path)s is being set with open permissions: '
-                            '%(perm)s'), {'path': path, 'perm': permissions})
+            LOG.warning('%(path)s is being set with open permissions: '
+                        '%(perm)s', {'path': path, 'perm': permissions})
 
         self._execute('chmod', permissions, path,
                       run_as_root=self._execute_as_root)
@@ -500,8 +546,8 @@ class RemoteFSDriver(driver.BaseVD):
                     share_opts = share_info[1].strip()
 
                 if not re.match(self.SHARE_FORMAT_REGEX, share_address):
-                    LOG.error(_LE("Share %s ignored due to invalid format. "
-                                  "Must be of form address:/export."),
+                    LOG.error("Share %s ignored due to invalid format. "
+                              "Must be of form address:/export.",
                               share_address)
                     continue
 
@@ -554,7 +600,7 @@ class RemoteFSDriver(driver.BaseVD):
     def _get_capacity_info(self, share):
         raise NotImplementedError()
 
-    def _find_share(self, volume_size_in_gib):
+    def _find_share(self, volume):
         raise NotImplementedError()
 
     def _ensure_share_mounted(self, share):
@@ -577,20 +623,20 @@ class RemoteFSDriver(driver.BaseVD):
         NAS file operations. This base method will set the NAS security
         options to false.
         """
-        doc_html = "http://docs.openstack.org/admin-guide" \
-                   "/blockstorage_nfs_backend.html"
+        doc_html = ("http://docs.openstack.org/admin-guide"
+                    "/blockstorage_nfs_backend.html")
         self.configuration.nas_secure_file_operations = 'false'
-        LOG.warning(_LW("The NAS file operations will be run as root: "
-                        "allowing root level access at the storage backend. "
-                        "This is considered an insecure NAS environment. "
-                        "Please see %s for information on a secure NAS "
-                        "configuration."),
+        LOG.warning("The NAS file operations will be run as root: "
+                    "allowing root level access at the storage backend. "
+                    "This is considered an insecure NAS environment. "
+                    "Please see %s for information on a secure NAS "
+                    "configuration.",
                     doc_html)
         self.configuration.nas_secure_file_permissions = 'false'
-        LOG.warning(_LW("The NAS file permissions mode will be 666 (allowing "
-                        "other/world read & write access). This is considered "
-                        "an insecure NAS environment. Please see %s for "
-                        "information on a secure NFS configuration."),
+        LOG.warning("The NAS file permissions mode will be 666 (allowing "
+                    "other/world read & write access). This is considered "
+                    "an insecure NAS environment. Please see %s for "
+                    "information on a secure NFS configuration.",
                     doc_html)
 
     def _determine_nas_security_option_setting(self, nas_option, mount_point,
@@ -614,8 +660,8 @@ class RemoteFSDriver(driver.BaseVD):
             file_path = os.path.join(mount_point, file_name)
             if os.path.isfile(file_path):
                 nas_option = 'true'
-                LOG.info(_LI('Cinder secure environment '
-                             'indicator file exists.'))
+                LOG.info('Cinder secure environment '
+                         'indicator file exists.')
             else:
                 # The indicator file does not exist. If it is a new
                 # installation, set to 'true' and create the indicator file.
@@ -631,11 +677,11 @@ class RemoteFSDriver(driver.BaseVD):
                         # protect from accidental removal (owner write only).
                         self._execute('chmod', '640', file_path,
                                       run_as_root=self._execute_as_root)
-                        LOG.info(_LI('New Cinder secure environment indicator'
-                                     ' file created at path %s.'), file_path)
+                        LOG.info('New Cinder secure environment indicator'
+                                 ' file created at path %s.', file_path)
                     except IOError as err:
-                        LOG.error(_LE('Failed to created Cinder secure '
-                                      'environment indicator file: %s'),
+                        LOG.error('Failed to created Cinder secure '
+                                  'environment indicator file: %s',
                                   err)
                 else:
                     # For existing installs, we default to 'false'. The
@@ -653,6 +699,11 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
     """
 
     _VALID_IMAGE_EXTENSIONS = []
+    # The following flag may be overriden by the concrete drivers in order
+    # to avoid using temporary volume snapshots when creating volume clones,
+    # when possible.
+
+    _always_use_temp_snap_when_cloning = True
 
     def __init__(self, *args, **kwargs):
         self._remotefsclient = None
@@ -699,17 +750,27 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
             msg = _("'active' must be present when writing snap_info.")
             raise exception.RemoteFSException(msg)
 
+        if not (os.path.exists(info_path) or os.name == 'nt'):
+            # We're not managing file permissions on Windows.
+            # Plus, 'truncate' is not available.
+            self._execute('truncate', "-s0", info_path,
+                          run_as_root=self._execute_as_root)
+            self._set_rw_permissions(info_path)
+
         with open(info_path, 'w') as f:
             json.dump(snap_info, f, indent=1, sort_keys=True)
 
-    def _qemu_img_info_base(self, path, volume_name, basedir):
+    def _qemu_img_info_base(self, path, volume_name, basedir,
+                            run_as_root=False):
         """Sanitize image_utils' qemu_img_info.
 
         This code expects to deal only with relative filenames.
         """
 
+        run_as_root = run_as_root or self._execute_as_root
+
         info = image_utils.qemu_img_info(path,
-                                         run_as_root=self._execute_as_root)
+                                         run_as_root=run_as_root)
         if info.image:
             info.image = os.path.basename(info.image)
         if info.backing_file:
@@ -726,7 +787,8 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                     'volname': volume_name,
                     'valid_ext': valid_ext,
                 }
-            if not re.match(backing_file_template, info.backing_file):
+            if not re.match(backing_file_template, info.backing_file,
+                            re.IGNORECASE):
                 msg = _("File %(path)s has invalid backing file "
                         "%(bfile)s, aborting.") % {'path': path,
                                                    'bfile': info.backing_file}
@@ -782,8 +844,9 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
             volume, active_file_path)
         higher_file = next((os.path.basename(f['filename'])
                             for f in backing_chain
-                            if f.get('backing-filename', '') ==
-                            snapshot_file),
+                            if utils.paths_normcase_equal(
+                                f.get('backing-filename', ''),
+                                snapshot_file)),
                            None)
         return higher_file
 
@@ -863,26 +926,6 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
     def _get_mount_point_base(self):
         return self.base
 
-    def _ensure_share_writable(self, path):
-        """Ensure that the Cinder user can write to the share.
-
-        If not, raise an exception.
-
-        :param path: path to test
-        :raises: RemoteFSException
-        :returns: None
-        """
-
-        prefix = '.cinder-write-test-' + str(os.getpid()) + '-'
-
-        try:
-            tempfile.NamedTemporaryFile(prefix=prefix, dir=path)
-        except OSError:
-            msg = _('Share at %(dir)s is not writable by the '
-                    'Cinder volume service. Snapshot operations will not be '
-                    'supported.') % {'dir': path}
-            raise exception.RemoteFSException(msg)
-
     def _copy_volume_to_image(self, context, volume, image_service,
                               image_meta):
         """Copy the volume to the specified image."""
@@ -932,16 +975,31 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
 
         return snap_info['active']
 
+    def _local_path_active_image(self, volume):
+        active_fname = self.get_active_image_from_info(volume)
+        vol_dir = self._local_volume_dir(volume)
+
+        active_fpath = os.path.join(vol_dir, active_fname)
+        return active_fpath
+
+    def _snapshots_exist(self, volume):
+        if not volume.provider_location:
+            return False
+
+        active_fpath = self._local_path_active_image(volume)
+        base_vol_path = self.local_path(volume)
+
+        return not utils.paths_normcase_equal(active_fpath, base_vol_path)
+
     def _create_cloned_volume(self, volume, src_vref):
-        LOG.info(_LI('Cloning volume %(src)s to volume %(dst)s'),
+        LOG.info('Cloning volume %(src)s to volume %(dst)s',
                  {'src': src_vref.id,
                   'dst': volume.id})
 
-        if src_vref.status not in ['available', 'backing-up']:
-            msg = _("Source volume status must be 'available', or "
-                    "'backing-up' but is: "
-                    "%(status)s.") % {'status': src_vref.status}
-            raise exception.InvalidVolume(msg)
+        acceptable_states = ['available', 'backing-up', 'downloading']
+        self._validate_state(src_vref.status,
+                             acceptable_states,
+                             obj_description='source volume')
 
         volume_name = CONF.volume_name_template % volume.id
 
@@ -950,10 +1008,6 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                      'volume_type', 'metadata']
         Volume = collections.namedtuple('Volume', vol_attrs)
 
-        snap_attrs = ['volume_name', 'volume_size', 'name',
-                      'volume_id', 'id', 'volume']
-        Snapshot = collections.namedtuple('Snapshot', snap_attrs)
-
         volume_info = Volume(provider_location=src_vref.provider_location,
                              size=src_vref.size,
                              id=volume.id,
@@ -962,24 +1016,38 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                              volume_type=src_vref.volume_type,
                              metadata=src_vref.metadata)
 
-        temp_snapshot = Snapshot(volume_name=volume_name,
-                                 volume_size=src_vref.size,
-                                 name='clone-snap-%s' % src_vref.id,
-                                 volume_id=src_vref.id,
-                                 id='tmp-snap-%s' % src_vref.id,
-                                 volume=src_vref)
+        if (self._always_use_temp_snap_when_cloning or
+                self._snapshots_exist(src_vref)):
+            snap_attrs = ['volume_name', 'volume_size', 'name',
+                          'volume_id', 'id', 'volume']
+            Snapshot = collections.namedtuple('Snapshot', snap_attrs)
 
-        self._create_snapshot(temp_snapshot)
-        try:
-            self._copy_volume_from_snapshot(temp_snapshot,
-                                            volume_info,
-                                            volume.size)
+            temp_snapshot = Snapshot(volume_name=volume_name,
+                                     volume_size=src_vref.size,
+                                     name='clone-snap-%s' % src_vref.id,
+                                     volume_id=src_vref.id,
+                                     id='tmp-snap-%s' % src_vref.id,
+                                     volume=src_vref)
 
-        finally:
-            self._delete_snapshot(temp_snapshot)
+            self._create_snapshot(temp_snapshot)
+            try:
+                self._copy_volume_from_snapshot(temp_snapshot,
+                                                volume_info,
+                                                volume.size)
+
+            finally:
+                self._delete_snapshot(temp_snapshot)
+        else:
+            self._copy_volume_image(self.local_path(src_vref),
+                                    self.local_path(volume_info))
+            self._extend_volume(volume_info, volume.size)
 
         return {'provider_location': src_vref.provider_location}
 
+    def _copy_volume_image(self, src_path, dest_path):
+        shutil.copyfile(src_path, dest_path)
+        self._set_rw_permissions(dest_path)
+
     def _delete_stale_snapshot(self, snapshot):
         info_path = self._local_path_volume_info(snapshot.volume)
         snap_info = self._read_info_file(info_path)
@@ -988,10 +1056,10 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
         active_file = self.get_active_image_from_info(snapshot.volume)
         snapshot_path = os.path.join(
             self._local_volume_dir(snapshot.volume), snapshot_file)
-        if (snapshot_file == active_file):
+        if utils.paths_normcase_equal(snapshot_file, active_file):
             return
 
-        LOG.info(_LI('Deleting stale snapshot: %s'), snapshot.id)
+        LOG.info('Deleting stale snapshot: %s', snapshot.id)
         self._delete(snapshot_path)
         del(snap_info[snapshot.id])
         self._write_info_file(info_path, snap_info)
@@ -1017,15 +1085,11 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                             else 'offline')})
 
         volume_status = snapshot.volume.status
-        if volume_status not in ['available', 'in-use', 'backing-up']:
-            msg = _("Volume status must be 'available', 'in-use' or "
-                    "'backing-up' but is: "
-                    "%(status)s.") % {'status': volume_status}
-
-            raise exception.InvalidVolume(msg)
+        acceptable_states = ['available', 'in-use', 'backing-up', 'deleting',
+                             'downloading']
+        self._validate_state(volume_status, acceptable_states)
 
         vol_path = self._local_volume_dir(snapshot.volume)
-        self._ensure_share_writable(vol_path)
 
         # Determine the true snapshot file for this snapshot
         # based on the .info file
@@ -1037,8 +1101,8 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
             # exist, do not attempt to delete.
             # (This happens, for example, if snapshot_create failed due to lack
             # of permission to write to the share.)
-            LOG.info(_LI('Snapshot record for %s is not present, allowing '
-                         'snapshot_delete to proceed.'), snapshot.id)
+            LOG.info('Snapshot record for %s is not present, allowing '
+                     'snapshot_delete to proceed.', snapshot.id)
             return
 
         snapshot_file = snap_info[snapshot.id]
@@ -1055,8 +1119,8 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
         if base_file is None:
             # There should always be at least the original volume
             # file as base.
-            LOG.warning(_LW('No backing file found for %s, allowing '
-                            'snapshot to be deleted.'), snapshot_path)
+            LOG.warning('No backing file found for %s, allowing '
+                        'snapshot to be deleted.', snapshot_path)
 
             # Snapshot may be stale, so just delete it and update the
             # info file instead of blocking
@@ -1077,7 +1141,8 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
 
             base_id = None
             for key, value in snap_info.items():
-                if value == base_file and key != 'active':
+                if utils.paths_normcase_equal(value,
+                                              base_file) and key != 'active':
                     base_id = key
                     break
             if base_id is None:
@@ -1097,7 +1162,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                                                 snapshot,
                                                 online_delete_info)
 
-        if snapshot_file == active_file:
+        if utils.paths_normcase_equal(snapshot_file, active_file):
             # There is no top file
             #      T0       |        T1         |
             #     base      |   snapshot_file   | None
@@ -1123,7 +1188,8 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                 raise exception.RemoteFSException(msg)
 
             higher_id = next((i for i in snap_info
-                              if snap_info[i] == higher_file
+                              if utils.paths_normcase_equal(snap_info[i],
+                                                            higher_file)
                               and i != 'active'),
                              None)
             if higher_id is None:
@@ -1158,7 +1224,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
 
         self._ensure_shares_mounted()
 
-        volume.provider_location = self._find_share(volume.size)
+        volume.provider_location = self._find_share(volume)
 
         self._do_create_volume(volume)
 
@@ -1327,12 +1393,15 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                             else 'offline')})
 
         status = snapshot.volume.status
-        if status not in ['available', 'in-use', 'backing-up']:
-            msg = _("Volume status must be 'available', 'in-use' or "
-                    "'backing-up' but is: "
-                    "%(status)s.") % {'status': status}
 
-            raise exception.InvalidVolume(msg)
+        acceptable_states = ['available', 'in-use', 'backing-up']
+        if snapshot.id.startswith('tmp-snap-'):
+            # This is an internal volume snapshot. In order to support
+            # image caching, we'll allow creating/deleting such snapshots
+            # while having volumes in 'downloading' state.
+            acceptable_states.append('downloading')
+
+        self._validate_state(status, acceptable_states)
 
         info_path = self._local_path_volume_info(snapshot.volume)
         snap_info = self._read_info_file(info_path, empty_if_missing=True)
@@ -1375,7 +1444,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                 connection_info)
             LOG.debug('nova call result: %s', result)
         except Exception:
-            LOG.exception(_LE('Call to Nova to create snapshot failed'))
+            LOG.exception('Call to Nova to create snapshot failed')
             raise
 
         # Loop and wait for result
@@ -1430,7 +1499,8 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
         info_path = self._local_path_volume_info(snapshot.volume)
         snap_info = self._read_info_file(info_path)
 
-        if info['active_file'] == info['snapshot_file']:
+        if utils.paths_normcase_equal(info['active_file'],
+                                      info['snapshot_file']):
             # blockRebase/Pull base into active
             # info['base'] => snapshot_file
 
@@ -1462,13 +1532,24 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
 
             del(snap_info[snapshot.id])
 
+        self._nova_assisted_vol_snap_delete(context, snapshot, delete_info)
+
+        # Write info file updated above
+        self._write_info_file(info_path, snap_info)
+
+        # Delete stale file
+        path_to_delete = os.path.join(
+            self._local_volume_dir(snapshot.volume), file_to_delete)
+        self._delete(path_to_delete)
+
+    def _nova_assisted_vol_snap_delete(self, context, snapshot, delete_info):
         try:
             self._nova.delete_volume_snapshot(
                 context,
                 snapshot.id,
                 delete_info)
         except Exception:
-            LOG.exception(_LE('Call to Nova delete snapshot failed'))
+            LOG.exception('Call to Nova delete snapshot failed')
             raise
 
         # Loop and wait for result
@@ -1508,14 +1589,8 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                     {'id': snapshot.id}
                 raise exception.RemoteFSException(msg)
 
-        # Write info file updated above
-        self._write_info_file(info_path, snap_info)
-
-        # Delete stale file
-        path_to_delete = os.path.join(
-            self._local_volume_dir(snapshot.volume), file_to_delete)
-        self._execute('rm', '-f', path_to_delete,
-                      run_as_root=self._execute_as_root)
+    def _extend_volume(self, volume, size_gb):
+        raise NotImplementedError()
 
 
 class RemoteFSSnapDriver(RemoteFSSnapDriverBase):
@@ -1548,6 +1623,10 @@ class RemoteFSSnapDriver(RemoteFSSnapDriverBase):
         return self._copy_volume_to_image(context, volume, image_service,
                                           image_meta)
 
+    @locked_volume_id_operation
+    def extend_volume(self, volume, size_gb):
+        return self._extend_volume(volume, size_gb)
+
 
 class RemoteFSSnapDriverDistributed(RemoteFSSnapDriverBase):
     @coordination.synchronized('{self.driver_prefix}-{snapshot.volume.id}')
@@ -1578,3 +1657,68 @@ class RemoteFSSnapDriverDistributed(RemoteFSSnapDriverBase):
 
         return self._copy_volume_to_image(context, volume, image_service,
                                           image_meta)
+
+    @coordination.synchronized('{self.driver_prefix}-{volume.id}')
+    def extend_volume(self, volume, size_gb):
+        return self._extend_volume(volume, size_gb)
+
+
+class RemoteFSPoolMixin(object):
+    """Drivers inheriting this will report each share as a pool."""
+
+    def _find_share(self, volume):
+        # We let the scheduler choose a pool for us.
+        pool_name = self._get_pool_name_from_volume(volume)
+        share = self._get_share_from_pool_name(pool_name)
+        return share
+
+    def _get_pool_name_from_volume(self, volume):
+        pool_name = volume_utils.extract_host(volume['host'],
+                                              level='pool')
+        return pool_name
+
+    def _get_pool_name_from_share(self, share):
+        raise NotImplementedError()
+
+    def _get_share_from_pool_name(self, pool_name):
+        # To be implemented by drivers using pools.
+        raise NotImplementedError()
+
+    def _update_volume_stats(self):
+        data = {}
+        pools = []
+        backend_name = self.configuration.safe_get('volume_backend_name')
+        data['volume_backend_name'] = backend_name or self.volume_backend_name
+        data['vendor_name'] = self.vendor_name
+        data['driver_version'] = self.get_version()
+        data['storage_protocol'] = self.driver_volume_type
+
+        self._ensure_shares_mounted()
+
+        for share in self._mounted_shares:
+            (share_capacity,
+             share_free,
+             share_used) = self._get_capacity_info(share)
+
+            pool = {'pool_name': self._get_pool_name_from_share(share),
+                    'total_capacity_gb': share_capacity / float(units.Gi),
+                    'free_capacity_gb': share_free / float(units.Gi),
+                    'provisioned_capacity_gb': share_used / float(units.Gi),
+                    'allocated_capacity_gb': (
+                        share_capacity - share_free) / float(units.Gi),
+                    'reserved_percentage': (
+                        self.configuration.reserved_percentage),
+                    'max_over_subscription_ratio': (
+                        self.configuration.max_over_subscription_ratio),
+                    'thin_provisioning_support': (
+                        self._thin_provisioning_support),
+                    'QoS_support': False,
+                    }
+
+            pools.append(pool)
+
+        data['total_capacity_gb'] = 0
+        data['free_capacity_gb'] = 0
+        data['pools'] = pools
+
+        self._stats = data
